/*
 * asteroid.cpp
 * 
 * Copyright 2012 Alexander Andrejevic <theflash@sdf.lonestar.org>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 */

#include "asteroid.h"
#include <ctime>
#include <cmath>
#include <cstdlib>
#include <cstring>

using namespace std;

asteroid::asteroid(int level, model3D *model, GLuint expl_tex)
{
	const GLfloat ambient_light[4]={0.0f, 0.0f, 0.0f, 0.0f};
	const GLfloat diffuse_light[4]={1.0f, 1.0f, 1.0f, 1.0f};
	const GLfloat specular_light[4]={1.0f, 1.0f, 1.0f, 1.0f};
	const GLfloat light_loc[4]={1.5f, 1.5f, 1.5f, 0.0f};
	const GLfloat material_ambient[4]={0.3f, 0.3f, 0.3f, 1.0f};
	const GLfloat material_diffuse[4]={0.8f, 0.8f, 0.8f, 1.0f};
	const GLfloat material_specular[4]={0.0f, 0.0f, 0.0f, 1.0f};
	const GLfloat material_emission[4]={0.0f, 0.0f, 0.0f, 1.0f};
	size=0.14f+(float)(rand()%2)/100.0f;
	strength=36.0f+(float)(rand()%5);
	z=-100.0f;
	x=(rand()%10)-5;
	y=(rand()%8)-4;
	g_level=level;
	g_model=model;
	rot_x=0.3f+(float)(rand()%7)/10.0f;
	rot_y=0.3f+(float)(rand()%7)/10.0f;
	rot_z=0.3f+(float)(rand()%7)/10.0f;
	delta_rot_x=0.03f+(float)(rand()%7)/100.0f;
	delta_rot_y=0.03f+(float)(rand()%7)/100.0f;
	delta_rot_z=0.03f+(float)(rand()%7)/100.0f;
	memcpy(lighting.ambient_light, ambient_light, 4*sizeof(GLfloat));
	memcpy(lighting.diffuse_light, diffuse_light, 4*sizeof(GLfloat));
	memcpy(lighting.specular_light, specular_light, 4*sizeof(GLfloat));
	memcpy(lighting.light_loc, light_loc, 4*sizeof(GLfloat));
	memcpy(material.material_ambient, material_ambient, 4*sizeof(GLfloat));
	memcpy(material.material_diffuse, material_diffuse, 4*sizeof(GLfloat));
	memcpy(material.material_specular, material_specular, 4*sizeof(GLfloat));
	memcpy(material.material_emission, material_emission, 4*sizeof(GLfloat));
	material.shininess=0.0f;
	exists=true;
	explosion=false;
	expl_texture=expl_tex;
}

asteroid::~asteroid()
{
}

void asteroid::draw()
{
	if(explosion)
	{
		glPushAttrib(GL_ENABLE_BIT | GL_DEPTH_BUFFER_BIT);
		glEnable(GL_TEXTURE_2D);
		glPushMatrix();
		glScalef(size, size, size);
		glTranslatef(x, y, z/size);
		glColor4f(1.0f, 1.0f, 1.0f, (float)timeout/600.0f);
		glEnable(GL_BLEND);
		glDisable(GL_DEPTH_TEST);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE);
		glBindTexture(GL_TEXTURE_2D, expl_texture);
		glBegin(GL_QUADS);
		    glTexCoord2f(0.0f, 0.0f); glVertex3f(-3.0f, -3.0f, 0.0f);
			glTexCoord2f(1.0f, 0.0f); glVertex3f(3.0f, -3.0f, 0.0f);
			glTexCoord2f(1.0f, 1.0f); glVertex3f(3.0f, 3.0f, 0.0f);
			glTexCoord2f(0.0f, 1.0f); glVertex3f(-3.0f, 3.0f, 0.0f);
		glEnd();
		glPopMatrix();
		glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
		if(!timeout) exists=false;
		glPopAttrib();
	}
	else
	{
		glPushAttrib(GL_ENABLE_BIT);
		glPushMatrix();
		glLightfv(GL_LIGHT0, GL_AMBIENT, lighting.ambient_light);
		glLightfv(GL_LIGHT0, GL_DIFFUSE, lighting.diffuse_light);
		glLightfv(GL_LIGHT0, GL_SPECULAR, lighting.specular_light);
		glLightfv(GL_LIGHT0, GL_POSITION, lighting.light_loc);
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, material.material_ambient);
		glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, material.material_diffuse);
		glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, material.material_specular);
		glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, material.material_emission);
		glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, material.shininess);
		glEnable(GL_LIGHT0);
		glDisable(GL_TEXTURE_2D);
		glEnable(GL_LIGHTING);
		glColor3f(0.1f, 0.0f, 0.0f);
		glLoadIdentity();
		g_model->draw_model(x, y, z, size, rot_x, rot_y, rot_z);
		glColor3f(1.0f, 1.0f, 1.0f);
		glPopMatrix();
		glPopAttrib();
	}
}

bool asteroid::move(int ms)
{
	if(explosion)
	{
		timeout-=ms;
		if(timeout<0) timeout=0;
		return true;
	}
	rot_x+=(GLfloat)ms*delta_rot_x;
	rot_y+=(GLfloat)ms*delta_rot_y;
	rot_z+=(GLfloat)ms*delta_rot_z;
	z+=(GLfloat)ms*(0.02f+(float)g_level*0.002f);
	return ((z/size)<=-2.0f) ? true:false;
}

bool asteroid::laser_hit(float intensity)
{
	strength-=intensity;
    if(strength<30.0f)
	{
	    material.material_emission[0]=(30.0f-strength)/30.0f;
		material.material_emission[1]=(30.0f-strength)/120.0f;
	}
	return (strength>0.0f)? false:true;
}

void asteroid::explode()
{
	explosion=true;
	timeout=600;
}
