/*
 * model3D.cpp
 * 
 * Copyright 2012 Alexander Andrejevic <theflash@sdf.lonestar.org>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 */

#include "model3D.h"
#include <cstdlib>
#include <cmath>
#include <cstdio>
#include <cstring>
#include <cassert>

model3D::model3D(const char *file_path)
{
	FILE *fd;
	char line[256];
	fd=fopen(file_path, "r");
	if(fd==NULL) throw "File Open Error";
	while(!feof(fd))
	{
		memset(line, 0, sizeof(line));
		fgets(line, 256, fd);
		if(line[0]=='#' || line[0]=='\n') continue;
	    if(strncmp(line, "v ", 2)==0)
		{
			GLfloat x, y, z;
			VERTEX v;
			sscanf(line, "v %f %f %f", &x, &y, &z);
			v.x=x;
			v.y=y;
			v.z=z;
			this->vertices.push_back(v);
		}
		else if(strncmp(line, "vn ", 3)==0)
		{
			GLfloat x, y, z;
		    NORMAL vn;
			sscanf(line, "vn %f %f %f", &x, &y, &z);
			vn.x=x;
			vn.y=y;
			vn.z=z;
			this->normals.push_back(vn);
		}
		else if(strncmp(line, "f ", 2)==0)
		{
			vector <pair <int, int> > face;
			char *copy=new char[strlen(line)];
			strcpy(copy, line+2);
			char *token=strtok(copy, " ");
			do
			{
				int v, n;
				sscanf(token, "%d//%d", &v, &n);
				face.push_back(make_pair(v, n));
				token=strtok(NULL, " ");
			}
			while(token!=NULL);
			faces.push_back(face);
			delete[] copy;
		}
	}
	fclose(fd);
	model_list=glGenLists(1);
	glNewList(this->model_list, GL_COMPILE);
    for(int i=0; i<faces.size(); i++)
	{
	    glBegin(GL_POLYGON);
	    for(int j=0; j<faces[i].size(); j++)
	    {
 		    VERTEX v=vertices[faces[i][j].first-1];
    		NORMAL n=normals[faces[i][j].second-1];
		    glNormal3f(n.x, n.y, n.z);
		    glVertex3f(v.x, v.y, v.z);
		}
		glEnd();
	}
	glEndList();
	vertices.clear();
	normals.clear();
	faces.clear();
}

model3D::~model3D()
{
	glDeleteLists(this->model_list, 1);
}

void model3D::draw_model(float x, float y, float z, float size, float rot_x, float rot_y, float rot_z)
{
	glPushMatrix();
	glEnable(GL_NORMALIZE);
	glLoadIdentity();
	glScalef(size, size, size);
	glTranslatef(x, y, z/size);
	glRotatef(rot_x, 1.0f, 0.0f, 0.0f);
	glRotatef(rot_y, 0.0f, 1.0f, 0.0f);
	glRotatef(rot_z, 0.0f, 0.0f, 1.0f);
	glCallList(this->model_list);
	glLoadIdentity();
	glDisable(GL_NORMALIZE);
	glPopMatrix();
}
