/*
 * tga_loader.cpp
 * 
 * Copyright 2012 Alexander Andrejevic <theflash@sdf.lonestar.org>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 */

#include "tga_loader.h"
#include <cstdlib>
#include <cstdio>

tga_image::tga_image(const char *file_path)
{
	FILE *fd=fopen(file_path, "rb");
	if(fd!=NULL)
	{
		fseek(fd, 0, SEEK_END);
		unsigned long size=ftell(fd);
		unsigned char *buffer=new unsigned char[size];
		rewind(fd);
		if(fread(buffer, 1, size, fd)==size)
		{
			TGA_HEADER *header=(TGA_HEADER*)buffer;
			if(header->image_type==10)
			{
				width=header->width;
				height=header->height;
				bpp=header->pixel_depth;
				int bytes_per_pixel=bpp/8;
				data=new unsigned char[width*height*(bpp/bytes_per_pixel)];
				unsigned char *source=buffer+sizeof(TGA_HEADER)+header->id_length;
				unsigned char *dest=data;
				unsigned long num_pixels=0;
				while(num_pixels<(width*height))
				{
					unsigned char rep_count=*(source++);
					unsigned char count=(rep_count&0x7F)+1;
					for(int i=0; i<count; i++)
					{
						for(int j=bytes_per_pixel-1; j>=0; j--) *(dest++)=*(source+j);
						if(!(rep_count&0x80)) source+=bytes_per_pixel;
					}
					if(rep_count&0x80) source+=bytes_per_pixel;
					num_pixels+=count;
				}
			}
			else throw "Invalid TGA File";
		}
		else throw "Read Error";
		delete[] buffer;
	}
	else throw "File Open Error";
}

tga_image::~tga_image()
{
	delete[] data;
}
